#include <assert.h>
#include <errno.h>
#include <limits.h>
#include <string.h>

#include "stdprj.h"
#include "machine.h"

void Machine::Input(Storage & S)
{
   // Exit on error!
   assert(S.GetSize() == GetSize());
   assert(IsHalted());

   // Transfer S to Memory
   int i;
   for (i=0; i<GetSize(); i++)
   {
      Memory[i] = S[i];
   }

   // Exit on error!
   assert(IsHalted());
}

Machine::Machine(int s) : Memory(s)
{
   // Exit on error!
   assert(s >= 1);

   // one char must have 8 bits
   assert(CHAR_BIT == 8);
}

Machine::~Machine()
{
}

int Machine::GetSize()
{
   return(Memory.GetSize());
}

void Machine::Load(const char * Path)
{
   int l;

   // Exit on error!
   assert(Path);
   l = strlen(Path);
   assert(errno == EZERO);
   assert(l > 0);
   assert(IsHalted());

   // load Memory with data from *Path
   Memory.Load(Path);

   // Exit on error!
   assert(IsHalted());
}

void Machine::Save(const char * Path)
{
   int l;

   // Exit bei on error!
   assert(Path);
   l = strlen(Path);
   assert(errno == EZERO);
   assert(l > 0);
   assert(IsHalted());

   // save data from Memory in into file *Path
   Memory.Save(Path);
}

#define RUNFLAG (Memory[0])

void Machine::Reset()
{
   // clear Memory
   Memory.Clear();

   // Machine is halted after Reset()
   RUNFLAG = FALSE;
}

void Machine::Run()
{
   // Exit on error!
   assert(IsHalted());

   // set RUNFLAG
   RUNFLAG = TRUE;
}

int Machine::IsHalted()
{
   // test RUNFLAG
   return(!RUNFLAG);
}

void Machine::Step()
{
   // Exit on error!
   assert(IsRunning());

   // halt Machine
   // set RUNFLAG
   RUNFLAG = FALSE;
   // Exit on error!
   assert(IsHalted());
}

void Machine::Run(Storage & S)
{
   // Exit on error!
   assert(S.GetSize() == GetSize());
   assert(IsHalted());

   // transfer S to Memory
   Input(S);
   // Exit on error!
   assert(IsHalted());

   Run();
}

int Machine::IsRunning()
{
   return(!(IsHalted()));
}
