#include <assert.h>
#include <errno.h>
#include <fstream.h>
#include <limits.h>
#include <string.h>

#include "stdprj.h"
#include "storage.h"

Storage::Storage(int s)
{
   // Exit on error!
   assert(s >= 1);

   // one char must have 8 bits
   assert(CHAR_BIT == 8);

   // remember amount of space
   Size = s;

   // allocate space from heap
   Byte = new unsigned char [Size];
   // Exit bei Fehler !
   assert(Byte);

   // clear Storage
   Clear();
}

Storage::~Storage()
{
   // free allocated space
   delete [] Byte;
}

int Storage::GetSize()
{
   return(Size);
}

void Storage::Clear()
{
   // clear Storage
   for (int i=0; i<Size; i++)
   {
      Byte[i] = 0;
   }
}

void Storage::Load(const char * Path)
{
   int c;
   int l;

   // Exit on error!
   assert(Path);
   l = strlen(Path);
   assert(errno == EZERO);
   assert(l > 0);

   // clear Storage
   Clear();

   // open file Path
   ifstream F(Path);
   // Exit on error!
   assert(errno == EZERO);

   // transfer data from F to Storage
   int s;
   F >> s;
   // Exit on error!
   assert(errno == EZERO);
   assert(Size == s);

   for (int i=0; i<Size; i++)
   {
      F >> c;
      // Exit on error!
      assert(errno == EZERO);
      assert(c >= 0);
      assert(c <= UCHAR_MAX);

      // transfer from F to Storage
      Byte[i] = (unsigned char) c;
   }
}

void Storage::Save(const char * Path)
{
   int l;

   // Exit on error!
   assert(Path);
   l = strlen(Path);
   assert(errno == EZERO);
   assert(l > 0);

   // open file Path
   ofstream F(Path);
   // Exit on error!
   assert(errno == EZERO);

   // transfer data from Storage to F
   F << (int) Size << endl;
   // Exit on error!
   assert(errno == EZERO);

   for (int i=0; i<Size; i++)
   {
      // formatted output
      F << "0x";
      // Exit on error!
      assert(errno == EZERO);

      F.width(2);
      // Exit on error!
      assert(errno == EZERO);

      F.fill('0');
      // Exit on error!
      assert(errno == EZERO);

      F.setf(F.uppercase);
      // Exit on error!
      assert(errno == EZERO);

      F << hex << (int) Byte[i];
      // Exit on error!
      assert(errno == EZERO);

      if (i < (Size - 1))
      {
         F << SPACE;
      }
      else
      {
         F << endl;
      }
      // Exit on error!
      assert(errno == EZERO);
   }
}

void Storage::FillFromArray(const unsigned char A[], int Length)
{
   // Exit on error!
   assert(A);
   assert(Length >= 1);
   assert(Size == Length);

   // transfer data from A to *this
   for (int i=0; i< Size; i++)
   {
      Byte[i] = A[i];
   }
}

void Storage::StoreToArray(unsigned char A[], int Length)
{
   // Exit on error!
   assert(A);
   assert(Length >= 1);
   assert(Size == Length);

   // transfer data from *this to A
   for (int i=0; i< Size; i++)
   {
      A[i] = Byte[i];
   }
}

unsigned char & Storage::operator[](int i)
{
   // Exit on error!
   assert(i >= 0);
   assert(i <  Size);

   return((this->Byte)[i]);
}

int Storage::operator==(Storage & S)
{
   // compare size
   if (Size != (S.GetSize())) return(FALSE);

   for (int i=0; i<Size; i++)
   {
      if (Byte[i] != (S.Byte)[i]) return(FALSE);
   }

   return(TRUE);
}

int Storage::operator!=(Storage & S)
{
   return(!((*this) == S));
}
